#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['NuRadioMC',
 'NuRadioMC.EvtGen',
 'NuRadioMC.SignalGen',
 'NuRadioMC.SignalGen.ARZ',
 'NuRadioMC.SignalProp',
 'NuRadioMC.SignalProp.CPPAnalyticRayTracing',
 'NuRadioMC.examples',
 'NuRadioMC.examples.02_DnR',
 'NuRadioMC.examples.Sensitivities',
 'NuRadioMC.simulation',
 'NuRadioMC.studies',
 'NuRadioMC.utilities']

package_data = \
{'': ['*'],
 'NuRadioMC': ['SignalGenProp/*',
               'data/*',
               'test/*',
               'test/SignalGen/*',
               'test/SignalProp/*',
               'test/SingleEvents/*',
               'test/Veff/*',
               'test/Veff/1e18eV/*',
               'test/examples/*'],
 'NuRadioMC.EvtGen': ['PROPOSAL_tables/*', 'tests/*'],
 'NuRadioMC.SignalGen': ['tests/*', 'tests/plots/*'],
 'NuRadioMC.SignalGen.ARZ': ['shower_library/*', 'tests/*'],
 'NuRadioMC.SignalProp': ['examples/*'],
 'NuRadioMC.examples': ['01_Veff_simulation/*',
                        '03_station_coincidences/*',
                        '04_read_ari_files/*',
                        '05_pulser_calibration_measurement/*',
                        'cluster/*'],
 'NuRadioMC.examples.02_DnR': ['detector/*', 'event_input/*'],
 'NuRadioMC.examples.Sensitivities': ['RNO_Proposal/*'],
 'NuRadioMC.simulation': ['detectors/*',
                          'detectors/ARA/*',
                          'detectors/depth/*']}

install_requires = \
['radiotools >=0.2.0', 'h5py', 'NuRadioReco >=1.0.0', 'pyyaml', 'cython']

setup(name='NuRadioMC',
      version='1.0.0',
      description='NuRadioMC: Simulating the radio emission of neutrinos from interaction to detector',
      author='Christian Glaser',
      author_email='work@c-glaser.de',
      url='https://github.com/nu-radio/NuRadioMC',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>3.4',
     )
